(*Title:  AnalyticGeometry*)

(*Context:  AnalyticGeometry`*)

(*Author:  harry Calkins*)

(*Contributions by Chetiya Sahabandu*)

(*Summary:  This package defines functions for creating classroom
examples using analytic geometry.*)

(*Package Version:  1.0*)

(*Mathematica Version: 5.2*)

(*Copyright: Copyright 2005, WolframResearch*)

(*History: Created August 2005*)

(*Last modified July 2011 for Mathematica 8.0.1*)

(*Keywords:  Distance, LinePointDistance, Parameterize, PerpendicularLine, PointSlopeLine,
TwoPointLine, PolygonInteriorQ*)






Distance::usage = "Distance[pt1,pt2] gives the Euclidean distance between pt1 and pt2 when the points are given in Cartesian coordinates."

TwoPointLine::usage = "TwoPointLine[pt1,pt2,var] gives an expression in var which is the point-slope representation of the line through pt1 and pt2."

PointSlopeLine::usage = "PointSlopeLine[pt1,slp,var] gives the slope-intercept form of the line through pt1 with slope m in terms of the variable var."

PerpendicularLine::usage = "PerpendicularLine[pt1,m,var] gives the slope-intercept form of the line through pt1 with slope -1/m in terms of the variable var."

IntermediatePoint::usage = "IntermediatePoint[pt1,pt2,pos] gives a point which is a weighted average of the two given points according to the formula (1-pos) pt1 + pos pt2. If pos is negative the order of the points will be newpt1, pt1,pt2, For 0 < pos < 1 the order will be pt1, newpt, pt2. For 1 < pos the order is pt1, pt2, newpt."

Slope::usage = "Slope[{pt1,pt2}] gives the slope of the line passing through the points pt1 and pt2."

Undefined::usage= "The value returned by Slope for vertical lines."

VerticalLines::vrtln = "The requested line through the point `1` is vertical: no expression will be generated for that line"

VerticalPoints::vrtln = "The requested line through the given points is a vertical line. No expression will be generated."

VerticalLineRepresentation::usage = "VerticalLineRepresentation is a option for PerpendicularLine and TwoPointLine that takes the values Equation, Automatic or None."

SecantLine::usage="SecantLine[fn,{var,varone,vartwo}] creates an expression for the secant line determined by the points varone and vartwo using the function fn. If varone equals vartwo the expression for the tangent line to fn at varone is given."

Equation::usage = "Equation is an option value for VerticalLineRepresentation."

PointLineDistance::usage = "PointLineDistance[pt,eq,var] finds the distance between the point pt and the line defined by eq."

PointLineSegment::usage = "PointLineSegment[pt,eq,var] gives the perpendicular line segment from pt to the line defined by eq."

MakeCircle::usage = "MakeCircle[cnt,rad,{var1,var2}] gives an expression for the circle with center cnt and radius rad as a function of var1 and var2.\nMakeCircle[pt1,pt2,pt3,{var1,var2}] gives and expression for the circle passing through the three points pt1, pt2 and pt3 as a function of var1 and var2."

SecantLinePlot::usage = "SecantLinePlot[f,{x,xmin,xmax},{x1,x2}] creates a plot of f with a secant line between {x1,f(x1)} and {x2,f(x2)}."

TangentLinePlot::usage = "TangentLinePlot[f,{x,xmin,xmax}, x1] creates a plot of f with a tangent line at the point {x1,f(x1)}.\nTangentLinePlot[{f,n},{x,xmin,xmax},x1] generates a plot of f and the first n derivatives of f and the tangent to f at the point {x1,f(x1)}.\nTangentLinePlot[{f,{n1,n2,...}},{x,xmin,xmax},x1] gives a plot of f with the specified derivatives of f and a tangent to f at the specified point."

TangentLine::usage = "TangentLine[f, x, pt] creates an expression for the tangent line to f at pt."

PointCurveDistance::usage = "PointCurveDistance[pt, expr, {x,strt}] gives the distance from the point pt to the curve defined byexpr starting with the value strt.]\nPointCurveDistance[pt,expr,x] automatically choses a start value."

PointCurvePlot::usage = "PointCurvePlot[expr,{x,xmin,xmax}, pt] returns a plot of expr on the interval xmin to xmax together with a circle tangent to expr centered on pt."

StartValue::usage = "StartValue is an option for PointCurvePlot to give a specific start value."

PolygonInteriorQ::usage = "PolygonInteriorQ[pt, poly] returns true if the point pt is in the interior of the polygon poly and false otherwise."

Unprotect[Distance,Equation, IntermediatePoint, TangentLine,TangentLinePlot,SecantLinePlot,
	MakeCircle,PointLineDistance, PointLineSegment, PointCurveDistance,PointCurvePlot,
	 PerpendicularLine, PointSlopeLine,SecantLine, Slope, TwoPointLine, Undefined,
	 VerticalLineRepresentation, PolygonInteriorQ]


Begin["`Private`"]

	
linearTest[ln_, var_Symbol]:=D[ln,var]=!=0 && D[ln,{var,2}]==0


IntermediatePoint[(pt1_)?VectorQ, (pt2_)?VectorQ, (pp_)?NumericQ] /; Length[pt1] == Length[pt2] := 
  Plus[(1 - pp)*pt1 + pp*pt2]

Distance[pt1_List, pt2_List] /; Length[pt1] == 
    Length[pt2] := Sqrt[Total[(pt1 - pt2)^2]]
    
Distance[lis_/;Dimensions[lis]==={2,2}]:= Distance@@lis
  
  
Slope[{pt1_ /; VectorQ[pt1, NumericQ], {x_Symbol, y_}}] := Divide @@ Reverse[
      pt1 - {x, y}];
      
      
Slope[{pt1_ /; VectorQ[pt1, NumericQ], pt2_ /; VectorQ[pt2, NumericQ]}] := 
      Module[{diff1, diff2}, {diff1, diff2} = pt1 -
           pt2; If[diff1 == 0, Undefined, diff2/diff1]];
           
    
    
Options[TwoPointLine] = {VerticalLineRepresentation -> Equation};


TwoPointLine[{x1_, y1_}, {x2_, y2_}, var_Symbol, 
    opts:OptionsPattern[]] := Module[{vrtlnrp},
    vrtlnrp = OptionValue[VerticalLineRepresentation];
    Which[(x1 == 
    x2 && vrtlnrp === None), Message[VerticalPoints::vrtln]; \
StringForm["TwoPointLine[``,``,``]", {x1, y1}, {x2, y2}, var],
      (x1 == x2 && vrtlnrp === Automatic), {var, x1},
      (x1 == x2 && vrtlnrp === Equation), Equal[var, x1],
      x1 =!= x2, (var*(y1 - y2))/(x1 - x2) + ((-x2)*y1 + x1*y2)/(x1 - x2)]
    ];
    
PointSlopeLine[{x1_, y1_}, Infinity|\[Infinity], var_Symbol] := 
 var==x1
  
  
PointSlopeLine[{x1_, y1_}, m_, var_Symbol] := 
  m*var - m*x1 + y1;
  
  

Options[PerpendicularLine] = {VerticalLineRepresentation -> Equation};

PerpendicularLine[pt_?VectorQ, expr_Times, var_Symbol,opts:OptionsPattern[]] := 
 Block[{slp = Coefficient[expr, var]}, 
  PerpendicularLine[pt, slp, var,{opts}]]
  
PerpendicularLine[pt_?VectorQ, expr_Plus, var_Symbol,opts:OptionsPattern[]] := 
 Block[{slp = Coefficient[expr, var]}, 
  PerpendicularLine[pt, slp, var,{opts}]]
  
PerpendicularLine[pt_?VectorQ, expr_Equal, var_Symbol,opts:OptionsPattern[]] := 
 Block[{ex = Cases[expr, exp_ /; Not[FreeQ[exp, var]]], slp},
  slp = Coefficient[ex[[1]], var];
  PerpendicularLine[pt, slp, var,{opts}]]
  
PerpendicularLine[pt_?VectorQ, zro:Equal[var_,num_?NumericQ]|Equal[num_?NumericQ,var_], var_Symbol,opts:OptionsPattern[]] := 
pt[[2]]
  
PerpendicularLine[pt_?VectorQ, zro:Equal[_,num_?NumericQ]|Equal[num_?NumericQ,_], var_Symbol,opts:OptionsPattern[]] := 
 Block[{slp=0},
  PerpendicularLine[pt, slp, var,{opts}]]

PerpendicularLine[{x1_, y1_}, m_, var_, opts:OptionsPattern[]] := Module[{vrtlnrp},
    vrtlnrp = OptionValue[VerticalLineRepresentation];
    Which[(m == 0 && vrtlnrp === None), Message[VerticalLines::vrtln, {x1, y1}];
           StringForm["PerpendicularLine[``,``,``]", {x1, y1}, m, var],
      m == 0 && vrtlnrp === Automatic, {var, x1},
      m == 0 && vrtlnrp=== Equation, Equal[var,x1],
      m =!= 0, PointSlopeLine[{x1, y1}, -m^(-1), var]]
    ];
 
 (*TODO Add additional definitions for PerpendicularLine to include expressions*)   
    
SecantLine[fn_, {var_Symbol, varone_, vartwo_}] := Module[{wrkfn, wrkpt},
  wrkfn[tt_] = 
   If[Head[fn] === Symbol && fn =!= var, fn[tt], fn /. var :> tt];
  wrkpt[tt_] = {tt, wrkfn[tt]};
  Piecewise[{{PointSlopeLine[wrkpt[varone], D[wrkfn[tt], tt] /. tt :> varone, 
      var], varone == vartwo}, {TwoPointLine[wrkpt[varone], wrkpt[vartwo], 
      var], varone =!= vartwo}}]
  ];
   
   
(*LinePointDistance*)

(*Danny thought of this test.*)
   
linearTest[ln_, var_Symbol] := 
 D[ln, var] =!= 0 && D[ln, {var, 2}] == 0 
   
(*This is the vertical line case.*)

PointLineDistance[pt_ /; VectorQ[pt, NumericQ], eq:Equal[ var_,num_?NumericQ]|Equal[num_?NumberQ,var_], 
  var_Symbol] := 
   Abs[pt[[1]]-num]
   
(*This is the horizontal line case*)

PointLineDistance[pt_ /; VectorQ[pt, NumericQ], eq:Equal[_,num_?NumericQ]|Equal[num_?NumberQ,_], 
  var_Symbol] := 
   Abs[pt[[2]]-num]
   
(*This is the general case with the slope and intercept given as values.*)
    
PointLineDistance[pt_ /; VectorQ[pt, NumericQ], eq_Equal, 
  var_Symbol] := 
 Block[{newln = PerpendicularLine[pt, eq, var], 
   ln = (Cases[eq, exp_ /; Not[FreeQ[exp, var]]]), sol},
  sol = Solve[newln == ln[[1]], var][[1]];
  Distance[pt, {var, ln[[1]]} /. sol]
  ] 
  
PointLineSegment[pt_ /; VectorQ[pt, NumericQ], eq:Equal[_,num_?NumericQ]|Equal[num_?NumberQ,_], 
  var_Symbol] := 
   Line[{pt,{pt[[1]],num}}]  
  
PointLineSegment[pt_ /; VectorQ[pt, NumericQ], eq_Equal, 
  var_Symbol] := 
 Block[{newln = PerpendicularLine[pt, eq, var], 
   ln = (Cases[eq, exp_ /; Not[FreeQ[exp, var]]]), sol},
  sol = Solve[newln == ln[[1]], var][[1]];
  Line[{pt, {var, ln[[1]]} /. sol}]
  ]   
  
    
(*These are the equivalent functions to generate the line segment from the
line to the point.*)
    
    

(*LinePointSegment[Infinity, xint_, (pt_)?VectorQ /; Length[pt] == 2] := 
  Line[{pt, {xint, pt[[2]]}}];
  
  

LinePointSegment[m_ /; m =!= Infinity, b_, {x0_, y0_}] := 
  Module[{sol}, sol = Solve[((a*{1, m} + {0, b}) - {x0, y0}) . {1, m} == 0, a]; 
    Line[{a*{1, m} + {0, b}, {x0, y0}} /. sol[[1]]]];
    
    

LinePointSegment[ln_Plus, (pt_)?VectorQ /; Length[pt] == 2, var_Symbol] /; 
    linearTest[ln, var] := LinePointSegment[Coefficient[ln, var], ln[[1]], pt]; 
    
    

LinePointSegment[ln_Times, (pt_)?VectorQ /; Length[pt] == 2, var_Symbol] /; 
   linearTest[ln, var] := LinePointSegment[Coefficient[ln, var], 0, pt];
   
   

LinePointSegment[var_Symbol, (pt_)?VectorQ /; Length[pt] == 2, var_Symbol] := 
   LinePointSegment[1, 0, pt]; 
   
   

LinePointSegment[(ln_)?NumericQ, (pt_)?VectorQ /; Length[pt] == 2, var_Symbol] := 
	LinePointSegment[0, ln, pt]; 
  
  

LinePointDistance[ln_,pt_,var_]/;Message[LinePointDistance::nonln,ln,var]:= Null;



LinePointSegment[ln_,pt_,var_]/;Message[LinePointSegment::nonln,ln,var]:= Null;



LinePointDistance[ln___]/;Message[LinePointDistance::argrx,LinePointDistance,Length[{ln}],3]:=Null;



LinePointSegment[ln___]/;Message[LinePointSegment::argrx,LinePointDistance,Length[{ln}],3]:=Null;



LinePointDistance::nonln = "Your expression `1` is not a linear function of `2`";



LinePointSegment::nonln = "Your expression `1` is not a linear function of `2`";*)

(*The circle functions*)

collinearTst[pt1_, pt2_, pt3_] := 
 Slope[{pt1, pt2}] != Slope[{pt1, pt3}]/.Undefined-> Infinity
 
MakeCircle::cllnr = "The given points are collinear."

MakeCircle[{x_, y_}, rad_, {var1_Symbol, 
  var2_Symbol}] := (x - var1)^2 + (y - var2)^2 - rad^2
  
MakeCircle[{x1_, y1_}, {x2_, y2_}, {x3_, y3_}, {var1_Symbol, 
   var2_Symbol}] /; (x1 == x2 && (y3 == y1 || y3 == y2)) := 
 Block[{cent = {Mean[{x1, x3}], 
     Pick[{Mean[{y1, y3}], Mean[{y2, y3}]}, {y1 != y3, 
        y != y3}][[1]]}, rad}, rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]]
  
MakeCircle[{x1_, y1_},{x3_, y3_},{x2_, y2_}, {var1_Symbol, 
   var2_Symbol}] /; (x1 == x2 && (y3 == y1 || y3 == y2)) := 
 Block[{cent = {Mean[{x1, x3}], 
     Pick[{Mean[{y1, y3}], Mean[{y2, y3}]}, {y1 != y3, 
        y != y3}][[1]]}, rad}, rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]]
  
MakeCircle[{x3_, y3_}, {x2_, y2_}, {x1_, y1_}, {var1_Symbol, 
   var2_Symbol}] /; (x1 == x2 && (y3 == y1 || y3 == y2)) := 
 Block[{cent = {Mean[{x1, x3}], 
     Pick[{Mean[{y1, y3}], Mean[{y2, y3}]}, {y1 != y3, 
        y != y3}][[1]]}, rad}, rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]]
  
MakeCircle[{x1_, y1_}, {x2_, y2_}, {x3_, y3_}, {var1_Symbol, 
   var2_Symbol}] /; (x1 == x2) := Block[{xx, sol, cent, rad},
  sol = Solve[
     Mean[{y1, y2}] == 
      PerpendicularLine[Mean[{{x1, y1}, {x3, y3}}], 
       Slope[{{x1, y1}, {x3, y3}}], xx], xx][[1]];
  cent = {xx, Mean[{y2, y1}]} /. sol;
  rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]
  ]
  
MakeCircle[{x1_, y1_},  {x3_, y3_},{x2_, y2_}, {var1_Symbol, 
   var2_Symbol}] /; (x1 == x2) := Block[{xx, sol, cent, rad},
  sol = Solve[
     Mean[{y1, y2}] == 
      PerpendicularLine[Mean[{{x1, y1}, {x3, y3}}], 
       Slope[{{x1, y1}, {x3, y3}}], xx], xx][[1]];
  cent = {xx, Mean[{y2, y1}]} /. sol;
  rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]
  ]
  
MakeCircle[{x3_, y3_},{x1_, y1_}, {x2_, y2_},  {var1_Symbol, 
   var2_Symbol}] /; (x1 == x2) := Block[{xx, sol, cent, rad},
  sol = Solve[
     Mean[{y1, y2}] == 
      PerpendicularLine[Mean[{{x1, y1}, {x3, y3}}], 
       Slope[{{x1, y1}, {x3, y3}}], xx], xx][[1]];
  cent = {xx, Mean[{y2, y1}]} /. sol;
  rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]
  ]
  
MakeCircle[{x1_, y1_}, {x2_, y2_}, {x3_, y3_}, {var1_Symbol, 
   var2_Symbol}] /; (y1 == y2) := Block[{xx, sol, cent, rad},
  sol = Solve[
     Mean[{x1, x2}] == 
      PerpendicularLine[Mean[{{x1, y1}, {x3, y3}}], 
       Slope[{{x1, y1}, {x3, y3}}], xx], xx][[1]];
  cent = {Mean[{x2, x1}], xx} /. sol;
  rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]
  ]
  
MakeCircle[{x1_, y1_},  {x3_, y3_},{x2_, y2_}, {var1_Symbol, 
   var2_Symbol}] /; (y1 == y2) := Block[{xx, sol, cent, rad},
  sol = Solve[
     Mean[{x1, x2}] == 
      PerpendicularLine[Mean[{{x1, y1}, {x3, y3}}], 
       Slope[{{x1, y1}, {x3, y3}}], xx], xx][[1]];
  cent = {Mean[{x2, x1}], xx} /. sol;
  rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]
  ]
  
MakeCircle[{x3_, y3_},{x1_, y1_}, {x2_, y2_},  {var1_Symbol, 
   var2_Symbol}] /; (y1 == y2) := Block[{xx, sol, cent, rad},
  sol = Solve[
     Mean[{x1, x2}] == 
      PerpendicularLine[Mean[{{x1, y1}, {x3, y3}}], 
       Slope[{{x1, y1}, {x3, y3}}], xx], xx][[1]];
  cent = {Mean[{x2, x1}], xx} /. sol;
  rad = Distance[cent, {x1, y1}];
  MakeCircle[cent, rad, {var1, var2}]
  ]

MakeCircle[pt1_List, pt2_List, pt3_List, {var1_Symbol, var2_Symbol}]/;collinearTst[pt1,pt2,pt3] :=
 
 Block[{mean1 = Mean[{pt1, pt2}], mean2 = Mean[{pt2, pt3}], 
   slp1 = Slope[{pt1, pt2}], slp2 = Slope[{pt2, pt3}], ln1, ln2, sol, 
   cent, rad, xx},
  ln1 = PerpendicularLine[mean1, slp1, xx];
  ln2 = PerpendicularLine[mean2, slp2, xx];
  sol = Solve[ln1 == ln2, xx][[1]];
  cent = {xx, ln1} /. sol;
  rad = Distance[pt1, cent];
  MakeCircle[cent, rad, {var1, var2}]]
  
MakeCircle[pt1_List, pt2_List, pt3_List,{var1_Symbol,var2_Symbol}]/;Not[collinearTst[pt1,pt2,pt3]] := (Message[MakeCircle::cllnr];
   StringForm["MakeCircle[`1`,`2`,`3`,{`4`,`5`}]",pt1,pt2,pt3,var1,var2])
   
TangentLine[fun_, var_Symbol, xloc : _?NumericQ | _Symbol] /; 
  xloc =!= var := Block[{der = D[fun, var], slp, pt},
  slp = der /. {var -> xloc};
  pt = {var, fun} /. {var -> xloc};
Simplify[PointSlopeLine[pt, slp, var]]]
  
TangentLine[fun_, var_Symbol, xloc_/;VectorQ[xloc,NumericQ]] := 
Block[{der = D[fun, var], slp, param,pt},
  slp = der /. {var -> param};
  pt = {var, fun} /. {var -> param};
  Simplify[Table[PointSlopeLine[pt,slp,var],{param,xloc}]]]
  
Options[TangentLinePlot]= {AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> True, 
 AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
 Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
 ClippingStyle -> None, ColorFunction -> Automatic, 
 ColorFunctionScaling -> True, ColorOutput -> Automatic, 
 ContentSelectable -> Automatic, CoordinatesToolOptions -> Automatic, 
 Epilog -> {}, Evaluated -> Automatic, EvaluationMonitor -> None, 
 Exclusions -> Automatic, ExclusionsStyle -> None, Filling -> None, 
 FillingStyle -> Automatic, Frame -> False, FrameLabel -> None, 
 FrameStyle -> {}, FrameTicks -> Automatic, FrameTicksStyle -> {}, 
 GridLines -> None, GridLinesStyle -> {}, ImageMargins -> 0., 
 ImagePadding -> All, ImageSize -> Automatic, 
 ImageSizeRaw -> Automatic, LabelStyle -> {}, 
 MaxRecursion -> Automatic, Mesh -> None, MeshFunctions -> {#1 &}, 
 MeshShading -> None, MeshStyle -> Automatic, Method -> Automatic, 
 PlotLabel -> None, PlotPoints -> Automatic, 
 PlotRange -> {Full, Automatic}, PlotRangeClipping -> True, 
 PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
 PlotStyle -> Automatic, PointStyle -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, 
 RegionFunction -> (True &), RotateLabel -> True, Ticks -> Automatic, 
 TicksStyle -> {}, WorkingPrecision -> MachinePrecision, 
 DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm, 
 PerformanceGoal :> $PerformanceGoal}


TangentLinePlot[
  fun:Except[_List], {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  xloc_?NumericQ, opts : OptionsPattern[]] := 
 Block[{der = D[fun, var], pt, ln,ptstyl},
  pt = {var, fun} /. {var -> xloc};
  ln = TangentLine[fun,var,xloc];
  ptstyl=(OptionValue[PointStyle]/.Automatic->{RGBColor[1,0,0],PointSize[Medium]});
  Show[{Plot[{fun, ln}, {var, varmin, varmax}, 
  	Evaluate@FilterRules[{opts},Options[Plot]],
  	Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]], 
    Graphics[Flatten[{ptstyl, Point[pt]}]]}]]
    
TangentLinePlot[
  fun:Except[_List], {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  xloc_/;VectorQ[xloc,NumericQ], opts : OptionsPattern[]] := 
 Block[{der = D[fun, var], pt, ln,ptstyl},
  pt = ({var, fun} /. List/@Thread[var -> xloc]);
  ln = TangentLine[fun,var,xloc];
  ptstyl=(OptionValue[PointStyle]/.Automatic->{RGBColor[1,0,0],PointSize[Medium]});
  Show[{Plot[{fun, ln}, {var, varmin, varmax}, 
  	Evaluate@FilterRules[{opts},Options[Plot]],
  	Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]], 
    Graphics[Flatten[{ptstyl, Point[pt]}]]}]]

TangentLinePlot[{fun_, int_Integer}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, xloc_?NumericQ, opts : OptionsPattern[]] := 
 Block[{der = Table[D[fun, {var, i}], {i, int}], pt, ln,ptstyl},
  pt = {var, fun} /. {var -> xloc};
  ln = TangentLine[fun,var,xloc];
  ptstyl=(OptionValue[PointStyle]/.Automatic->{RGBColor[1,0,0],PointSize[Medium]});
  Show[{Plot[{fun, Sequence @@ der, ln}, {var, varmin, varmax}, 
  	Evaluate@FilterRules[{opts},Options[Plot]],
  	Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]],
     Graphics[Flatten[{ptstyl, Point[pt]}]]}]]
     
TangentLinePlot[{fun_, int_Integer}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, xloc_/;VectorQ[xloc,NumericQ], opts : OptionsPattern[]] := 
 Block[{der = Table[D[fun, {var, i}], {i, int}], pt, ln, ptstyl}, 
  pt = {var, fun} /. List/@Thread[var -> xloc];
  ln = TangentLine[fun, var, xloc];
  ptstyl = (OptionValue[PointStyle] /. 
     Automatic -> {RGBColor[1, 0, 0], PointSize[Medium]});
  Show[{Plot[{fun, Sequence @@ der, ln}, {var, varmin, varmax}, 
     Evaluate@FilterRules[{opts}, Options[Plot]], 
     Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]], 
    Graphics[Flatten[{ptstyl, Point[pt]}]]}]]
     
TangentLinePlot[{fun_, int_ /; VectorQ[int, IntegerQ]}, {var_Symbol, 
   varmin_?NumericQ, varmax_?NumericQ}, xloc_?NumericQ, 
  opts : OptionsPattern[]] := 
 Block[{der = Table[D[fun, {var, i}], {i, int}], pt, ln,ptstyl},
  pt = {var, fun} /. {var -> xloc};
  ln = TangentLine[fun,var,xloc];
  ptstyl=(OptionValue[PointStyle]/.Automatic->{RGBColor[1,0,0],PointSize[Medium]});
  Show[{Plot[
     Evaluate@{fun, Sequence @@ der, ln}, {var, varmin, varmax}, 
     Evaluate@FilterRules[{opts},Options[Plot]],
     Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]], 
     Graphics[Flatten[{ptstyl, Point[pt]}]]}]]

TangentLinePlot[{fun_, int_ /; VectorQ[int, IntegerQ]}, {var_Symbol, 
   varmin_?NumericQ, varmax_?NumericQ}, xloc_/;VectorQ[xloc,NumericQ], 
  opts : OptionsPattern[]] := 
 Block[{der = Table[D[fun, {var, i}], {i, int}], pt, ln,ptstyl},
  pt = Transpose[({var, fun} /. {var -> xloc})];
  ln = TangentLine[fun,var,xloc];
  ptstyl=(OptionValue[PointStyle]/.Automatic->{RGBColor[1,0,0],PointSize[Medium]});
  Show[{Plot[
     Evaluate@{fun, Sequence @@ der, ln}, {var, varmin, varmax}, 
     Evaluate@FilterRules[{opts},Options[Plot]],
     Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]], 
     Graphics[Flatten[{ptstyl, Point[pt]}]]}]]
     
     
Options[SecantLinePlot]= {AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> True, 
 AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
 Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
 ClippingStyle -> None, ColorFunction -> Automatic, 
 ColorFunctionScaling -> True, ColorOutput -> Automatic, 
 ContentSelectable -> Automatic, CoordinatesToolOptions -> Automatic, 
 Epilog -> {}, Evaluated -> Automatic, EvaluationMonitor -> None, 
 Exclusions -> Automatic, ExclusionsStyle -> None, Filling -> None, 
 FillingStyle -> Automatic, Frame -> False, FrameLabel -> None, 
 FrameStyle -> {}, FrameTicks -> Automatic, FrameTicksStyle -> {}, 
 GridLines -> None, GridLinesStyle -> {}, ImageMargins -> 0., 
 ImagePadding -> All, ImageSize -> Automatic, 
 ImageSizeRaw -> Automatic, LabelStyle -> {}, 
 MaxRecursion -> Automatic, Mesh -> None, MeshFunctions -> {#1 &}, 
 MeshShading -> None, MeshStyle -> Automatic, Method -> Automatic, 
 PlotLabel -> None, PlotPoints -> Automatic, 
 PlotRange -> {Full, Automatic}, PlotRangeClipping -> True, 
 PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
 PlotStyle -> Automatic, PointStyle -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, 
 RegionFunction -> (True &), RotateLabel -> True, Ticks -> Automatic, 
 TicksStyle -> {}, WorkingPrecision -> MachinePrecision, 
 DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm, 
 PerformanceGoal :> $PerformanceGoal}

SecantLinePlot[
  fun_, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, {xloc1_?NumericQ, xloc2_?NumericQ}, 
  opts : OptionsPattern[]] := Block[{der = D[fun, var], pt, ln,ptstyl},
  slp = der /. {var -> xloc};
  pt = {var, fun} /. {{var -> xloc1}, {var -> xloc2}};
  ln = SecantLine[fun, {var, xloc1, xloc2}];
  ptstyl=(OptionValue[PointStyle]/.Automatic->{RGBColor[1,0,0],PointSize[Medium]});
  Show[{Plot[{fun, ln}, {var, varmin, varmax}, Evaluate@FilterRules[{opts},Options[Plot]],
  	Evaluate[FilterRules[Options[SecantLinePlot], Options[Plot]]]], 
    Graphics[Flatten[{ptstyl, Point[pt]}]]}]]
    
(*Point to curve functions*)


PointCurveDistance[pt_ /; VectorQ[pt, NumericQ], 
  expr_, {var_Symbol, strt_?NumericQ}] := 
 Block[{mindis = 
    FindMinimum[EuclideanDistance[pt, {var, expr}], {var, strt}]},
  {mindis[[1]], {var, expr} /. mindis[[2]]}]

PointCurveDistance[pt_ /; VectorQ[pt, NumericQ], expr_, var_Symbol] := 
 Block[{mindis = FindMinimum[EuclideanDistance[pt, {var, expr}], var]},
  {mindis[[1]], {var, expr} /. mindis[[2]]}]
  
strtval[pt_, expr_, var_, Automatic] := 
 PointCurveDistance[pt, expr, var]

strtval[pt_, expr_, var_, num_?NumericQ] := 
 PointCurveDistance[pt, expr, {var, num}]
 
Options[PointCurvePlot] = {AccuracyGoal -> Automatic, 
   AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> True, 
   AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
   Background -> None, BaselinePosition -> Automatic, BaseStyle -> {},
    ClippingStyle -> None, ColorFunction -> Automatic, 
   ColorFunctionScaling -> True, ColorOutput -> Automatic, 
   Compiled -> Automatic, ContentSelectable -> Automatic, 
   CoordinatesToolOptions -> Automatic, Epilog -> {}, 
   Evaluated -> Automatic, EvaluationMonitor -> None, 
   Exclusions -> Automatic, ExclusionsStyle -> None, Filling -> None, 
   FillingStyle -> Automatic, Frame -> False, FrameLabel -> None, 
   FrameStyle -> {}, FrameTicks -> Automatic, FrameTicksStyle -> {}, 
   Gradient -> Automatic, GridLines -> None, GridLinesStyle -> {}, 
   ImageMargins -> 0.`, ImagePadding -> All, ImageSize -> Automatic, 
   ImageSizeRaw -> Automatic, LabelStyle -> {}, 
   MaxIterations -> Automatic, MaxRecursion -> Automatic, 
   Mesh -> None, MeshFunctions -> {#1 &}, MeshShading -> None, 
   MeshStyle -> Automatic, Method -> Automatic, PlotLabel -> None, 
   PlotPoints -> Automatic, PlotRange -> All, 
   PlotRangeClipping -> True, PlotRangePadding -> Automatic, 
   PlotRegion -> Automatic, PlotStyle -> Automatic, 
   PointStyle -> Automatic, PrecisionGoal -> Automatic, 
   PreserveImageOptions -> Automatic, Prolog -> {}, 
   RegionFunction -> (True &), RotateLabel -> True, 
   StartValue -> Automatic, StepMonitor -> None, Ticks -> Automatic, 
   TicksStyle -> {}, WorkingPrecision -> MachinePrecision, 
   DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm,
    PerformanceGoal :> $PerformanceGoal};
 
PointCurvePlot[
  expr_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  pt_ /; VectorQ[pt, NumericQ], opts : OptionsPattern[]] := 
 Block[{mindis, ptstyl},
  mindis = 
   strtval[pt, expr, var, 
    OptionValue[PointCurvePlot, {opts}, StartValue]];
  ptstyl = 
   OptionValue[PointCurvePlot, {opts}, PointStyle] /. 
    Automatic -> {Orange, PointSize[Medium]};
  Show[{Plot[expr, {var, varmin, varmax}, 
     Evaluate[FilterRules[{opts}, Options[Plot]]]], 
    Graphics[
     Flatten[{ptstyl, Point[{pt, mindis[[2]]}], 
       Circle[pt, mindis[[1]]]}]]}, 
   FilterRules[{opts}, Options[Graphics]],
   FilterRules[Options[PointCurvePlot], Options[Graphics]]]]
   
(*The function PolygonInteriorQ is used to determine if a point is in the interior
of a polygon. It was developed by Chetiya Sahabandu.*)

(*BoundingBoxQ returns True iff the point p is in the interior of the bounding rectangle of the polygon or on its boundary; Returns False otherwise.*)

boundingBoxQ[p_, 
  vert_] := (Min[vert[[All, 1]]] <= p[[1]] <= 
    Max[vert[[All, 1]]]) && (Min[vert[[All, 2]]] <= p[[2]] <= 
    Max[vert[[All, 2]]])

(*Select the edges that are intersected by a horizontal line extending from the point p. *)

selectEdges[e_, p_] := 
  Module[{s, res}, 
   s = Cases[e, 
     t : {{x1_, y1_}, {x2_, 
          y2_}} /; ((y1 <= p[[2]]) && (y2 >= p[[2]])) || ((y1 >= 
            p[[2]]) && (y2 <= p[[2]])) :> t];
   res = s;
   res];

(*For a point P={x0, y0} and a line L={{x1, y1}, {x2, y2}} leftOfLineQ returns Indeterminate if L is horizontal, 1 if P is on the LHS of L, -1 if P is on the RHS of L, and 0 if P is on L.*)

leftOfLineQ[{x0_, 
    y0_}, {{x1_, y1_}, {x2_, y2_}}] /; (y1 != y2) && (x1 != x2) := 
 Module[{xtest = Min[{x1, x2}] - 1, ytest = Mean[{y1, y2}], 
   gradient = (y2 - y1)/(x2 - x1), intercept},
  intercept = y1 - gradient*x1;
  Sign[(gradient*x0 + intercept - y0)*(gradient*xtest + intercept - ytest)]]

leftOfLineQ[{x0_, 
    y0_}, {{x1_, y1_}, {x2_, y2_}}] /; (y1 != y2) && (x1 == x2) := 
 Module[{xtest = Min[{x1, x2}] - 1, ytest = Mean[{y1, y2}]}, 
  Sign[(x0 - x1)*(xtest - x1)]]

leftOfLineQ[___] := Indeterminate

(*This is the user function*)

PolygonInteriorQ[p : {x0_, y0_}, vert_] := 
 Module[{edges, selectedEdges, pick, intersectingEdges, edgeIntersectionCount,
    horizontalEdgeCount, vertexCrossingCount, exceptions, res},
  
  res = Catch[If[boundingBoxQ[p, vert],
     
     edges = Partition[vert, 2, 1];
     selectedEdges = selectEdges[edges, p];
     pick = leftOfLineQ[p, #] & /@ selectedEdges;
     If[MemberQ[pick, 0], Throw[False]];
     intersectingEdges = Pick[selectedEdges, pick, -1];
     edgeIntersectionCount = Length@intersectingEdges;
     horizontalEdgeCount = 
      Count[edgeIntersectionCount, {{_, x_}, {_, y_}} /; x == y];
     vertexCrossingCount = Count[Flatten[intersectingEdges, 1], {_, p[[2]]}]/2;
     exceptions = vertexCrossingCount - horizontalEdgeCount;
     OddQ[edgeIntersectionCount - exceptions],
     False,
     $Failed]]
  ]
  
PolygonInteriorQ[pts : {{_?NumericQ, _?NumericQ} ..}, vert_List] := 
 Block[{res = PolygonInteriorQ[#, vert] & /@ pts},
  Transpose[{pts, res}]]  
   
    
End[]

SetAttributes[{Distance,Equation, IntermediatePoint, TangentLine,TangentLinePlot,SecantLinePlot,
	MakeCircle,PointLineDistance, PointLineSegment, PointCurveDistance, PointCurvePlot,
	 PerpendicularLine, PointSlopeLine,SecantLine, Slope, TwoPointLine, Undefined,
	 VerticalLineRepresentation, PolygonInteriorQ},{Protected,ReadProtected}] 

