(* Mathematica Package *)

(* Created by the Wolfram Workbench Jan 1, 2010 *)



Trajectory::usage = "Trajectory[t,{vel,ang,g,k}] gives the position at time t in parametric coordinates of an object with initial position {0, 0}, initial velocity vel, initial direction given by the angle ang with the horizontal and subject to air resistance k velocity squared."

MPH::usage = "MPH gives the number of feet per second traveled at one mile per hour. It has a numerical value of \!\(5280\/\[NonBreakingSpace]60\^2\)."

KPH::usage ="KPH gives the number of meters per second traveled at one kilometer per hour. It has a numerical value of \!\(1000\/\[NonBreakingSpace]60\^2\)."

CubicBezier::usage = "CubicBezier[var,{pt1,pt2,pt3,pt4}] creates a parametric representation of a cubic bezier curve with anchor points pt1 and pt4 and control points pt2 and pt3."

PartitionSum::usage = "PartitionSum[sum,n] separates the first n terms from the rest of the sum."

ChangeIndices::usage = "ChangeIndices[sum, index, newindex] adjusts the index set in the series from indes to new index."

GetSummands::usage = "GetSummands[summation] extracts the summands from the summation."

MakeSeries::usage = "MakeSeries[x, a, n, ord] creates a series summation for of the form Sum[ a[n]\!\(x \^ n\), {n,0,Infinity}] together with derivative of the series up to order ord."

RealSolve::usage = "ReSolve[eq,var] gives all real valued solutions for the equations eq."

CubicBezierComponents::usage = "CubicBezierComponents[t,{pt1,pt2,pt3,pt4}] genrates the function pair used in a parametric representation of a cubic bezier curve with anchors pt1 and pt4 and control points pt2 and pt3."

Unprotect[CubicBezierComponents,CubicBezier,RealSolve,MakeSeries,GetSummands,ChangeIndices,PartitionSum,MPH,KPH,Trajectory]
(* Exported symbols added here with SymbolName::usage *) 

Begin["`Private`"]
(* Trajectory functions *)

MPH = 5280/60^2

KPH = 1000/60^2

(*These functions, unfortunately give drag as proportional to velocity.*)

(*Trajectory[t_, {vel_, ang_, g_, k_}] := {(
  vel Cos[ang] - E^(-2 k t) vel Cos[ang])/(2 k), (
  g - E^(-2 k t) g - 2 g k t + 2 k vel Sin[ang] - 
   2 E^(-2 k t) k vel Sin[ang])/(4 k^2)}

Trajectory[t_, {vel_, ang_, g_, k_},"Horizontal"] := (
  vel Cos[ang] - E^(-2 k t) vel Cos[ang])/(2 k)
  
Trajectory[t_, {vel_, ang_, g_, k_},"Vertical"] :=  (
  g - E^(-2 k t) g - 2 g k t + 2 k vel Sin[ang] - 
   2 E^(-2 k t) k vel Sin[ang])/(4 k^2)*)
(*These are the functions with no air resistance*)

Trajectory[t_, {vel_, ang_, g_}] := {t vel Cos[ang], 
  1/2 (-g t^2 + 2 t vel Sin[ang])}
  
Trajectory[t_, {vel_, ang_, g_},"Horizontal"] := t vel Cos[ang]

Trajectory[t_, {vel_, ang_, g_},"Vertical"] :=  
   1/2 (-g t^2 + 2 t vel Sin[ang])
  
(*Four parameter forms of no air resistance.*)

Trajectory[t_, {vel_, ang_, g_,0|0.0}] := {t vel Cos[ang], 
  1/2 (-g t^2 + 2 t vel Sin[ang])}
  
Trajectory[t_, {vel_, ang_, g_,0|0.0},"Horizontal"] := t vel Cos[ang]

Trajectory[t_, {vel_, ang_, g_,0|0.0},"Vertical"] :=  
  1/2 (-g t^2 + 2 t vel Sin[ang])
   
(*these are the functions with drag proportional to the square of velocity.*)
   
Trajectory[t_, {vel_, ang_, g_, k_}] := {(-Log[Sec[ang]/vel] + Log[k t + Sec[ang]/vel])/k, (
 Log[Cos[Sqrt[g] Sqrt[k] t - 
     ArcTan[(Sqrt[k] vel Sin[ang])/Sqrt[g]]]] + 
  1/2 Log[1 + (k vel^2 Sin[ang]^2)/g])/k}
  
Trajectory[t_, {vel_, ang_, g_, k_},"Horizontal"] := (-Log[Sec[ang]/vel] + Log[k t + Sec[ang]/vel])/k
  
Trajectory[t_, {vel_, ang_, g_, k_},"Vertical"] :=  (
 Log[Cos[Sqrt[g] Sqrt[k] t - 
     ArcTan[(Sqrt[k] vel Sin[ang])/Sqrt[g]]]] + 
  1/2 Log[1 + (k vel^2 Sin[ang]^2)/g])/k
  

   
(*Bezier functions with symbolic representaion *)

CubicBezier[t_, {pt1_, pt2_, pt3_, pt4_}] := \[Piecewise] {
   {Simplify[(1 - t)^3 pt1 + 3 (1 - t)^2 t pt2 + 3 (1 - t) t^2 pt3 + t^3 pt4], 
    0 <= t <= 1},
   {{Indeterminate, Indeterminate}, True}
  }
  
CubicBezierComponents[
  t_, {pt1_, pt2_, pt3_, pt4_}] := Simplify[(1 - t)^3 pt1 + 
  3 (1 - t)^2 t pt2 + 3 (1 - t) t^2 pt3 + t^3 pt4] 
  
CubicBezierComponents[t_, {pt1_, pt2_, pt3_, pt4_}, "Horizontal"] := 
 CubicBezierComponents[t, {pt1, pt2, pt3, pt4}][[1]]

CubicBezierComponents[t_, {pt1_, pt2_, pt3_, pt4_}, "Vertical"] := 
 CubicBezierComponents[t, {pt1, pt2, pt3, pt4}][[2]]
  
(*Functions for series solutions of ODEs*)

PartitionSum[Sum[expr_, {var_, varmin_, varmax_}], 
   varint_] /; (varmin <= varint < varmax) := 
 Sum[expr, {var, varmin, varint-1}] + 
  Sum[expr, {var, varint , varmax}]

ChangeIndices[expr : Sum[trm_, {var_, varmin_, varmax_}], indx_, 
  newindx_] := 
 With[{newtrm = Simplify[trm /. indx -> newindx], 
   newstrt = varmin + indx - newindx}, 
  Sum[newtrm, {var, newstrt, varmax}]]

GetSummands[sum_Sum] := sum[[1]]

GetSummands[Unevaluated[sums : Plus[_Sum ..]]] := 
 Block[{sumlis = (List @@ sums)},
  Together[Total[Map[GetSummands[#] &, sumlis]]]]
  
MakeSeries[var_Symbol, a_Symbol, iter_Symbol, ord_Integer?NonNegative] := 
 (Map[sum[#, {iter, 0, Infinity}] &, 
   Table[D[a[iter] Power[var, iter], {var, deg}], {deg, 0, ord}]] )/. 
  {sum -> Sum}
  

RealSolve[eqs_Equal, solvar : _Symbol | _List] := 
 Extract[Solve[eqs, solvar], 
 Position[N@Solve[eqs, solvar], ({x_Rule..} /; FreeQ[x, Complex])]]
   

End[]

SetAttributes[{CubicBezierComponents,CubicBezier,RealSolve,MakeSeries,GetSummands,ChangeIndices,PartitionSum,MPH,KPH,Trajectory},{Protected,ReadProtected}]



