(* Mathematica package *)

Shell::usage = "Shell[{r1,r2,btm,tp}] construct a washer with inner radius r1, outer radius r2 and thickness tp-btm."

RiemannSumPlot3D::usage = "RiemannSumPlot3D[f,{var,varmin,varmax},n,\"type\"] creates n partitions of \"type\" to approximate the volume obtained by revolving the function f about the vertical axis."

MakeRadiusFunction::usage = "MakeRadiusFunction[fnlis,{var,varmin,varmax}] creates an interpolating function for the inverses of the functions in fnlis."

VerticalSegment::usage = "VerticalSegment[int,{val,ineq}] creates a vertical line segment on the interval int for use in MakeRadiusFunction."

MakeVerticalSegment::usage = "MakeVerticalSegment[val,{var,varmin,varmax}] creates a VerticalSegment for use in RiemannSum3D and RiemannSumPlot3D."

CapStyle::usage = "CapStyle is an option for RiemannSumPlot that controls the appearance of the flat surfaces of shells and washers."

RiemannSum3D::usage = "RiemannSum3D[f,{x,xmin, xmax},n, \"Disk\"]"

Begin["Private`"]  (*Begin private context*)

(*============================  Tests  ========================================*)

finitTst[lis_List, {var_Symbol, varmin_, varmax_}] := 
 FreeQ[Map[
   Quiet[Abs[Maximize[{#, varmin < var < varmax}, x][[1]]] < 
      Infinity] &, Flatten[lis]], False]

(*===============================  Disks  ======================================*)
(*This assumes the radius of the cylinder is determined by the function value at 
the midpoint of the length of the cylinder. *)

invFun[fun_, {var_, varmin_, varmax_}] := 
 Block[{funPrs, stp = (varmax - varmin)/100}, 
  funPrs = Table[{Function[var, fun][var2], var2}, {var2, 
     Range[varmin, varmax, stp]}];
  Interpolation[funPrs, Method -> "Spline"]]

vertFunPrs[fun_InterpolatingFunction, n_Integer?Positive] := 
 Block[{stpV = 
    Differences[fun[[1, 1]]][[1]]/n}, {stpV, {#, fun[#]} & /@ 
    Range[fun[[1, 1, 1]] + stpV/2, fun[[1, 1, 2]], stpV]}]

vertFunPrs[{fun_, fun2_}, {var_, varmin_, varmax_, stp_}] := 
 Table[{fun, fun2, var - stp/2, var + stp/2}, {var, varmin + stp/2, 
   varmax, stp}]

vertFunPrs[{minval_, {fun_, fun2_}}, {var_, varmin_, varmax_, stp_}] :=
  Table[{minval, Min[{fun, fun2}], var - stp/2, var + stp/2}, {var, 
   varmin + stp/2, varmax, stp}]

vertFunPrs[{{fun_, fun2_}, maxval_}, {var_, varmin_, varmax_, stp_}] :=
  Table[{maxval, Max[{fun, fun2}], var - stp/2, var + stp/2}, {var, 
   varmin + stp/2, varmax, stp}]

shllFun[{k_, r_}, stp_?Positive] := {0, r, k - stp, k + stp}

makeDisks[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, opts : OptionsPattern[]] :=
 Block[{stp = (varmax - varmin)/100, stpV, funPrsV, newFn},
  newFn = invFun[fun, {var, varmin, varmax}];
  {stpV, funPrsV} = vertFunPrs[newFn, n];
  Cylinder[({{0, 0, #1 - stpV/2}, {0, 0, #1 + stpV/2}} & /@ 
     funPrsV[[All, 1]]), funPrsV[[All, 2]]]
  ]

makeDisks[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, ang_, 
  opts : OptionsPattern[CapStyle -> Automatic]] :=
 Block[{stp = (varmax - varmin)/100, stpV, funPrsV, newFn, 
   cpStyl = OptionValue[CapStyle]},
  newFn = invFun[fun, {var, varmin, varmax}];
  {stpV, funPrsV} = vertFunPrs[newFn, n];
  Shell[shllFun[#, stpV/2] & /@ funPrsV, ang, CapStyle -> cpStyl]
  ]
  
(*====================================  Shells  ==================================*)

closest[val_?NumericQ] := 
 With[{vals = Range[0, 2 \[Pi] - (\[Pi]/36), \[Pi]/36], 
   val2 = Mod[N@val, 2 \[Pi]]},
  Position[Abs[vals - val2], Min[Abs[vals - val2]]]]

makePts[lis1_List, lis2_List, r_?NumericQ, k_?NumericQ] := 
 Transpose[{r Cos[lis1], r Sin[lis1], k lis2}]

Options[Shell] = {CapStyle -> GrayLevel[.75]};

Shell[{r_, r_, __}, ___] := {}

Shell[{r1_?NumericQ, r2_, k1_, k2_}, opts : OptionsPattern[]] /; 
  r1 != r2 := 
 With[{lis1 = Range[0., 2 \[Pi] - \[Pi]/36, \[Pi]/36], 
   lis2 = ConstantArray[1., 72], 
   rimPos = 
    Join[Flatten[
      Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 
          218}} + i, {i, 0, 70}], 
      1], {{144, 72, 1}, {1, 144, 73}, {288, 216, 145}, {145, 288, 
       217}}], flatPts1 = 
    Join[Range[72], {1, 145}, Range[216, 145, -1]], 
   cpstyl = (OptionValue[CapStyle] /. List -> Directive)}, 
  GraphicsComplex[
   Join[makePts[lis1, lis2, r1, k1], makePts[lis1, lis2, r1, k2], 
    makePts[lis1, lis2, r2, k1], 
    makePts[lis1, lis2, r2, k2]], {EdgeForm[], Polygon[rimPos], 
    cpstyl, Polygon[{flatPts1, flatPts1 + 72}]}]]

Shell[shls : {{_?NumericQ, _, _, _} ..}, opts : OptionsPattern[]] := 
 With[{ cpstyl = (OptionValue[CapStyle] /. List -> Directive)}, 
  Map[Shell[#, CapStyle -> cpstyl] &, shls]]

Shell[{r1_?NumericQ, r2_, k1_, 
    k2_}, {stp_Integer?Positive /; 1 <= stp < 72}, 
   opts : OptionsPattern[]] /; r1 != r2 := 
 With[{lis1 = Range[0., 2 \[Pi] - \[Pi]/36, \[Pi]/36], 
   lis2 = ConstantArray[1., 72], 
   rimPos = 
    Flatten[Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 
         217, 218}} + i, {i, 0, stp - 1}], 1], 
   flatPts1 = 
    Join[Take[Range[72], stp + 1], 
     Take[Range[216, 145, -1], -(stp + 1)]], 
   cpstyl = (OptionValue[CapStyle] /. List -> Directive)},
  GraphicsComplex[
   Join[makePts[lis1, lis2, r1, k1], makePts[lis1, lis2, r1, k2], 
    makePts[lis1, lis2, r2, k1], 
    makePts[lis1, lis2, r2, k2]], {EdgeForm[], Polygon[rimPos], 
    cpstyl, Polygon[{flatPts1, flatPts1 + 72}], 
    Polygon[{{1, 73, 217, 145}, {stp + 1, stp + 73, stp + 217, 
       stp + 145}}]}]]

Shell[shls : {{_?NumericQ, _, _, _} ..}, {stp_Integer?Positive /; 
    1 <= stp < 72}, opts : OptionsPattern[]] := 
 With[{cpstyl = (OptionValue[CapStyle] /. List -> Directive)},
  Map[Shell[#, {stp}, CapStyle-> cpstyl] &, shls]]

Shell[{r1_?NumericQ, r2_, k1_, k2_}, {0}, opts : OptionsPattern[]] /; 
  r1 != r2 := 
 With[{lis1 = Range[0., 2 \[Pi] - \[Pi]/36, \[Pi]/36], 
   lis2 = ConstantArray[1., 72],
   cpstyl = (OptionValue[CapStyle] /. List -> Directive)},
  GraphicsComplex[
   Join[makePts[lis1, lis2, r1, k1], makePts[lis1, lis2, r1, k2], 
    makePts[lis1, lis2, r2, k1], 
    makePts[lis1, lis2, r2, k2]], {EdgeForm[], cpstyl, 
    Polygon[{{1, 73, 217, 145}}]}]]

Shell[shls : {{_?NumericQ, _, _, _} ..}, {0}, 
  opts : OptionsPattern[]] := 
 (*With[{lis1 = Range[0., 2 \[Pi] - \[Pi]/36, \[Pi]/36], 
   lis2 = ConstantArray[1., 72, 
   cpstyl = (OptionValue[CapStyle] /. List -> Directive)},*)
  Map[Shell[#, {0}, opts] &, shls]

Shell[{r1_, r2_, k1_, k2_}, {72}, opts : OptionsPattern[]] /; 
  r1 != r2 := Shell[{r1, r2, k1, k2}, opts]

Shell[shls : {{_?NumericQ, _, _, _} ..}, {72}, 
  opts : OptionsPattern[]] := Shell[shls, opts]

Shell[{r1_, r2_, k1_, k2_}, nn_?Positive, opts : OptionsPattern[]] /; 
  r1 != r2 := Block[{rr = closest[nn][[1]] /. {1 -> 72}},
  Shell[{r1, r2, k1, k2}, rr, opts]]

Shell[shls : {{_?NumericQ, _, _, _} ..}, nn_?Positive, 
  opts : OptionsPattern[]] := 
 Block[{rr = closest[nn][[1]] /. {1 -> 72}},
  Shell[shls, rr, opts]]

Shell[{r1_, r2_, k1_, k2_}, 0, opts : OptionsPattern[]] /; r1 != r2 :=
  Shell[{r1, r2, k1, k2}, {0}, opts]

Shell[shls : {{_?NumericQ, _, _, _} ..}, 0, opts : OptionsPattern[]] :=
  Shell[shls, {0}, opts]

Shell[{r1_, r2_, k1_, k2_}, 0., opts : OptionsPattern[]] /; r1 != r2 :=
  Shell[{r1, r2, k1, k2}, {0}, opts]

Shell[shls : {{_?NumericQ, _, _, _} ..}, 0., 
  opts : OptionsPattern[]] := Shell[shls, {0}, opts]
  
(*================================  RiemannSumPlot3D  =============================*)

Options[RiemannSumPlot3D] = Options[riemannSumPlot3D]={AlignmentPoint -> Center, AspectRatio -> Automatic, 
 AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
 AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
 Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
 Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
 CapStyle -> {EdgeForm[GrayLevel[.65]],GrayLevel[0.4]}, ColorOutput -> Automatic, 
 ContentSelectable -> Automatic, ControllerLinking -> Automatic, 
 ControllerMethod -> Automatic, ControllerPath -> Automatic, 
 CoordinatesToolOptions -> Automatic, 
 DisplayFunction :> $DisplayFunction, Epilog -> {}, FaceGrids -> None,
  FaceGridsStyle -> {}, FormatType :> TraditionalForm, 
 ImageMargins -> 0., ImagePadding -> All, ImageSize -> Automatic, 
 LabelStyle -> {}, Lighting -> Automatic, Method -> Automatic, 
 PlotLabel -> None, PlotRange -> All, PlotRangePadding -> Automatic, 
 PlotRegion -> Automatic, PlotStyle -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, 
 RotationAction -> "Fit", SphericalRegion -> False, 
 Ticks -> Automatic, TicksStyle -> {}, ViewAngle -> Automatic, 
 ViewCenter -> Automatic, ViewMatrix -> Automatic, 
 ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> All, 
 ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}
 
(*=================================  Shell  ===================================*) 
 
RiemannSumPlot3D[
  f_, {var_Symbol, varmin_?NonNegative, varmax_?Positive}, n_Integer, 
  "Shell", opts : OptionsPattern[]] := 
 Block[{stp = (varmax - varmin)/n, 
   pltStyl = ( 
     OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}), 
   cpstyl = OptionValue[CapStyle]},
  Graphics3D[{pltStyl, 
    Shell[Table[{r, r + stp, 0, f /. var :> (r+stp/2)}, {r, varmin, 
       varmax - stp, stp}], CapStyle -> cpstyl]}, 
   FilterRules[{opts}, Options[Graphics3D]]]]

RiemannSumPlot3D[{f1_, f2_}, {var_Symbol, varmin_?NonNegative, 
   varmax_?Positive}, n_Integer, "Shell", opts : OptionsPattern[]] := 
 Block[{stp = (varmax - varmin)/n, 
   pltStyl = ( 
     OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}), 
   cpstyl = OptionValue[CapStyle]},
  Graphics3D[{pltStyl, 
    Shell[Table[{r, r + stp, f1 /. var :> (r+stp/2), f2 /. var :> (r+stp/2)}, {r, 
       varmin, varmax - stp, stp}], CapStyle -> cpstyl]}, 
   FilterRules[{opts}, Options[Graphics3D]]]]

RiemannSumPlot3D[
  f_, {var_Symbol, varmin_?NonNegative, varmax_?Positive}, {n_Integer, ang_?NumericQ}, 
  "Shell", opts : OptionsPattern[]] := 
 Block[{stp = (varmax - varmin)/n, 
   pltStyl = ( 
     OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}), 
   cpstyl = OptionValue[CapStyle]},
  Graphics3D[{pltStyl, 
    Shell[Table[{r, r + stp, 0, f /. var :> (r+stp/2)}, {r, varmin, 
       varmax - stp, stp}], ang, CapStyle -> cpstyl]}, 
   FilterRules[{opts}, Options[Graphics3D]]]]

RiemannSumPlot3D[{f1_, f2_}, {var_Symbol, varmin_?NonNegative, 
   varmax_?Positive}, {n_Integer, ang_?NumericQ}, "Shell", 
  opts : OptionsPattern[]] := 
 Block[{stp = (varmax - varmin)/n, 
   pltStyl = ( 
     OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}), 
   cpstyl = OptionValue[CapStyle]},
  Graphics3D[{pltStyl, 
    Shell[Table[{r, r + stp, f1 /. var :> (r+stp/2), f2 /. var :> (r+stp/2)}, {r, 
       varmin, varmax - stp, stp}], ang, CapStyle -> cpstyl]}, 
   FilterRules[{opts}, Options[Graphics3D]]]]
   
(*========================  Disk  =======================================*)   

RiemannSumPlot3D[
  f_, {var_Symbol, varmin_?NonNegative, varmax_?Positive}, n_Integer, 
  "Disk", opts : OptionsPattern[]] := 
 Block[{stp = (varmax - varmin)/n, 
   pltStyl = ( 
     OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}), 
   cpstyl = OptionValue[CapStyle]},
  Graphics3D[{pltStyl, makeDisks[f,{var,varmin,varmax}, n]}, 
   FilterRules[{opts}, Options[Graphics3D]]]]

RiemannSumPlot3D[
  f_, {var_Symbol, varmin_?NonNegative, varmax_?Positive}, {n_Integer, ang_?NumericQ}, 
  "Disk",  opts : OptionsPattern[]] := 
 Block[{stp = (varmax - varmin)/n, 
   pltStyl = ( 
     OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}), 
   cpstyl = OptionValue[CapStyle]},
  Graphics3D[{pltStyl, 
    makeDisks[f,{var,varmin,varmax}, n, ang, CapStyle -> cpstyl]}, 
   FilterRules[{opts}, Options[Graphics3D]]]]
   
(*====================RiemannSumPlot3D --  Washers  =====================*)   
   
validTst[lis_List] := OrderedQ[Flatten@(SortBy[lis, First])]
   
MakeVerticalSegment[
  val_?Positive, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}] := 
 VerticalSegment[{{varmin, varmax}}, {val, varmin <= var < varmax}]

GetFunctionDefinition[vrt_VerticalSegment] := vrt[[2]]  

makePcwsInt[{a0_, a1_}, var_Symbol] := \!\(\*
TagBox[
StyleBox[
RowBox[{"Inequality", "[", 
RowBox[{"a0", ",", "LessEqual", ",", "var", ",", "Less", ",", "a1"}], 
      "]"}],
ShowSpecialCharacters->False,
ShowStringCharacters->True,
NumberMarks->True],
FullForm]\)

interpInv[
  f1 : Except[_VerticalSegment], {var_Symbol, varmin_, varmax_}, 
  n_Integer: 50] := 
 Block[{valTbl = 
    Table[{f1, var}, {var, varmin, varmax, (varmax - varmin)/n}]}, 
  Interpolation[valTbl, Method -> "Spline"]]

interpInv[f1_VerticalSegment, {var_Symbol, varmin_, varmax_}] := 
 f1 /. f1[[2, 2, 3]] :> var
 
pcwsComponent[f1_InterpolatingFunction, 
  var_Symbol] := {f1[var], makePcwsInt[f1[[1, 1]], var]}

pcwsComponent[f1_VerticalSegment, var_Symbol] := 
 Block[{fun = GetFunctionDefinition[f1]},
  fun /. fun[[2, 3]] :> var]

 makePcws[
  lis : {(_InterpolatingFunction | _VerticalSegment) ..}(*/;validTst[
  lis]*), var_Symbol] := Piecewise[Map[pcwsComponent[#, var] &, lis]] 
  
MakeRadiusFunction::ovrlp = "Two or more of the inverse function \
support intervals for the `1` are not disjoint."
  
makeRadiusFunction[lis_List, 
  varlis : {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}] := 
 Block[{invrs = Map[interpInv[#, varlis] &, lis], res},
  If[validTst[invrs[[All, 1, 1]]],
   Function @@ (FullForm[(makePcws[invrs, var])] /. {var :> 
        Slot[1]}),
   $Failed]
  ] 
  
MakeRadiusFunction[lis_List,
	varlis:{var_Symbol,varmin_?NumericQ,varmax_?NumericQ}]:=   
  Block[{res= makeRadiusFunction[lis,varlis]},
  	If[res===$Failed,Message[MakeRadiusFunction::ovrlp,"radii"]];
  	res/;res=!= $Failed]
   
riemannSumPlot3D[{{f1__}, {f2__}}, varlis : {x_Symbol, x0_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer", 
  opts : OptionsPattern[]] := 
 Block[{res},
	res=Catch[
		Block[{stp = (z1 - z0)/n, shlpts, pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}),
  				cpstyl = OptionValue[CapStyle],fnInv1,fnInv2 , shlls},
  fnInv1 = makeRadiusFunction[{f1}, varlis];
  fnInv2 = makeRadiusFunction[{f2}, varlis]; 
  Which[fnInv1 === $Failed, Throw[#]&[{$Failed,1}],
   fnInv2 === $Failed,  Throw[{$Failed,2}],
   True, 
  shlpts = Table[{fnInv1[t+stp/2],fnInv2[t+stp/2],t,t+stp}, {t, z0, z1 - stp, stp}];
   Graphics3D[{pltStyl, Shell[shlpts, CapStyle -> cpstyl]}, 
    FilterRules[{opts}, Options[Graphics3D]]]]
    ]];
    res]   
   
riemannSumPlot3D[{f1_, f2_}, varlis : {x_Symbol, x0_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer", 
  opts : OptionsPattern[]] := 
   riemannSumPlot3D[{{f1},{f2}},varlis,rnglis,n,"Washer",opts]

riemannSumPlot3D[{{f1__}, {f2__}}, varlis : {x_Symbol, x0_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", opts : OptionsPattern[]] :=
Block[{res},
	res=Catch[
		Block[{stp = (z1 - z0)/n, shlpts, pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}),
  				cpstyl = OptionValue[CapStyle],fnInv1,fnInv2 , shlls},
  fnInv1 = makeRadiusFunction[{f1}, varlis];
  fnInv2 = makeRadiusFunction[{f2}, varlis]; 
  Which[fnInv1 === $Failed, Throw[#]&[{$Failed,1}],
   fnInv2 === $Failed,  Throw[{$Failed,2}],
   True, 
  shlpts = Table[{fnInv1[t+stp/2],fnInv2[t+stp/2],t,t+stp}, {t, z0, z1 - stp, stp}];
   Graphics3D[{pltStyl, Shell[shlpts,ang, CapStyle -> cpstyl]}, 
    FilterRules[{opts}, Options[Graphics3D]]]]
    ]];
    res]
   
riemannSumPlot3D[{f1_, f2_}, varlis : {x_Symbol, x0_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", opts : OptionsPattern[]] :=
    riemannSumPlot3D[{{f1},{f2}},varlis,rnglis,{n,ang},"Washer",opts] 
  
riemannSumPlot3D[{{f1__}, {f2__}}, {x_Symbol, x0_,xm_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer", 
  opts : OptionsPattern[]] := 
 Block[{res},
	res=Catch[
		Block[{stp = (z1 - z0)/n, shlpts, pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}),
  				cpstyl = OptionValue[CapStyle],fnInv1,fnInv2 , shlls},
  fnInv1 = makeRadiusFunction[{f1}, {x,x0,xm}];
  fnInv2 = makeRadiusFunction[{f2}, {x,xm,x1}]; 
  Which[fnInv1 === $Failed, Throw[#]&[{$Failed,1}],
   fnInv2 === $Failed,  Throw[{$Failed,2}],
   True, 
  shlpts = Table[{fnInv1[t+stp/2],fnInv2[t+stp/2],t,t+stp}, {t, z0, z1 - stp, stp}];
   Graphics3D[{pltStyl, Shell[shlpts, CapStyle -> cpstyl]}, 
    FilterRules[{opts}, Options[Graphics3D]]]]
    ]];
    res]
   
riemannSumPlot3D[{f1_, f2_}, varlis:{x_Symbol, x0_,xm_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer", 
  opts : OptionsPattern[]] := 
   riemannSumPlot3D[{{f1},{f2}},varlis,rnglis,n,"Washer",opts]    
   
riemannSumPlot3D[{{f1__}, {f2__}}, {x_Symbol, x0_,xm_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", 
  opts : OptionsPattern[]] := 
 Block[{res},
	res=Catch[
		Block[{stp = (z1 - z0)/n, shlpts, pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}),
  				cpstyl = OptionValue[CapStyle],fnInv1,fnInv2 , shlls},
  fnInv1 = makeRadiusFunction[{f1}, {x,x0,xm}];
  fnInv2 = makeRadiusFunction[{f2}, {x,xm,x1}]; 
  Which[fnInv1 === $Failed, Throw[#]&[{$Failed,1}],
   fnInv2 === $Failed,  Throw[{$Failed,2}],
   True, 
  shlpts = Table[{fnInv1[t+stp/2],fnInv2[t+stp/2],t,t+stp}, {t, z0, z1 - stp, stp}];
   Graphics3D[{pltStyl, Shell[shlpts,ang, CapStyle -> cpstyl]}, 
    FilterRules[{opts}, Options[Graphics3D]]]]
    ]];
    res]
   
riemannSumPlot3D[{f1_, f2_}, varlis:{x_Symbol, x0_,xm_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", 
  opts : OptionsPattern[]] := 
   riemannSumPlot3D[{{f1},{f2}},varlis,rnglis,{n,ang},"Washer",opts]      
   
riemannSumPlot3D[{{f1__}, {f2__}}, {x_Symbol, x0_,xm1_,xm2_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer", 
  opts : OptionsPattern[]] := 
 Block[{res},
	res=Catch[
		Block[{stp = (z1 - z0)/n, shlpts, pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}),
  				cpstyl = OptionValue[CapStyle],fnInv1,fnInv2 , shlls},
  fnInv1 = makeRadiusFunction[{f1}, {x,x0,xm1}];
  fnInv2 = makeRadiusFunction[{f2}, {x,xm2,x1}]; 
  Which[fnInv1 === $Failed, Throw[#]&[{$Failed,1}],
   fnInv2 === $Failed,  Throw[{$Failed,2}],
   True, 
  shlpts = Table[{fnInv1[t+stp/2],fnInv2[t+stp/2],t,t+stp}, {t, z0, z1 - stp, stp}];
   Graphics3D[{pltStyl, Shell[shlpts, CapStyle -> cpstyl]}, 
    FilterRules[{opts}, Options[Graphics3D]]]]
    ]];
    res]
   
riemannSumPlot3D[{f1_, f2_}, varlis:{x_Symbol, x0_,xm1_,xm2_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer", 
  opts : OptionsPattern[]] := 
   riemannSumPlot3D[{{f1},{f2}},varlis,rnglis,n,"Washer",opts]     
   
riemannSumPlot3D[{{f1__}, {f2__}}, {x_Symbol, x0_,xm1_,xm2_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", 
  opts : OptionsPattern[]] := 
 Block[{res},
	res=Catch[
		Block[{stp = (z1 - z0)/n, shlpts, pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, List -> Directive}),
  				cpstyl = OptionValue[CapStyle],fnInv1,fnInv2 , shlls},
  fnInv1 = makeRadiusFunction[{f1}, {x,x0,xm1}];
  fnInv2 = makeRadiusFunction[{f2}, {x,xm2,x1}]; 
  Which[fnInv1 === $Failed, Throw[#]&[{$Failed,1}],
   fnInv2 === $Failed,  Throw[{$Failed,2}],
   True, 
  shlpts = Table[{fnInv1[t+stp/2],fnInv2[t+stp/2],t,t+stp}, {t, z0, z1 - stp, stp}];
   Graphics3D[{pltStyl, Shell[shlpts,ang, CapStyle -> cpstyl]}, 
    FilterRules[{opts}, Options[Graphics3D]]]]
    ]];
    res]
   
riemannSumPlot3D[{f1_, f2_}, varlis:{x_Symbol, x0_,xm1_,xm2_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", 
  opts : OptionsPattern[]] :=
   riemannSumPlot3D[{{f1},{f2}},varlis,rnglis,{n,ang},"Washer",opts] 
   
RiemannSumPlot3D[fnLis_List, varlis_List, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, 
    "Washer", opts : OptionsPattern[]] :=
  Block[{res22 = 
    riemannSumPlot3D[fnLis, varlis, rnglis, n, "Washer", opts]},
  If[MatchQ[res22, {$Failed, 1 | 2}], 
   Message[MakeRadiusFunction::ovrlp, {"inner radius", 
      "outer radius"}[[res22[[2]]]]]];
  res22 /; FreeQ[res22, $Failed]]  
  
RiemannSumPlot3D[fnLis_List, varlis_List, 
  rnglis : {z_Symbol, z0_, z1_}, {n_Integer?Positive, 
   ang_ /; 0 <= ang <= 2 \[Pi]}, "Washer", opts : OptionsPattern[]] :=
  Block[{res22 = 
    riemannSumPlot3D[fnLis, varlis, rnglis, {n, ang}, "Washer", opts]},
  If[MatchQ[res22, {$Failed, 1 | 2}], 
   Message[MakeRadiusFunction::ovrlp, {"inner radius", 
      "outer radius"}[[res22[[2]]]]]];
  res22 /; FreeQ[res22, $Failed]]     
  
  (*=========================    RiemannSum3D  ==================================*)
  (*========================   For Disks  =======================================*)
  
RiemannSum3D[f_, varlis : {var_Symbol, varmin_, varmax_}, 
   n_Integer?Positive, "Disk"] /; finitTst[{f}, varlis] := 
 	Block[{stp = (varmax - varmin)/100, stpV, funPrsV, newFn},
  	  newFn = invFun[f, {var, varmin, varmax}];
      {stpV, funPrsV} = vertFunPrs[newFn, n];
      Total[\[Pi] stpV (funPrsV[[All, 2]])^2]
  ]
  
  (*========================   For Washers  =======================================*)

riemannSum3D[{f1_, f2_}, varlis : {x_Symbol, x0_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer"] := 
  RiemannSum3D[{{f1},{f2}},varlis,rnglis,n,"Washer"]

riemannSum3D[{{f1__}, {f2__}}, varlis : {x_Symbol, x0_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer"] := 
 Block[{res}, 
  res = Catch[
    Block[{stp = (z1 - z0)/n, shlpts}, 
     fnInv1 = makeRadiusFunction[{f1}, varlis];
     fnInv2 = makeRadiusFunction[{f2}, varlis];
     Which[fnInv1 === $Failed, Throw[#] &[{$Failed, 1}], 
      fnInv2 === $Failed, Throw[{$Failed, 2}], True, 
      shlpts = \[Pi] stp Sum[(-fnInv1[t + stp/2]^2 + 
           fnInv2[t + stp/2]^2), {t, z0, z1 - stp, stp}]]]];
  res]

riemannSum3D[{{f1__}, {f2__}}, {x_Symbol, x0_, xm_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer"] := 
 Block[{res}, 
  res = Catch[
    Block[{stp = (z1 - z0)/n, shlpts, 
      pltStyl = (OptionValue[PlotStyle] /. {Automatic -> {}, 
          List -> Directive}), cpstyl = OptionValue[CapStyle], fnInv1,
       fnInv2, shlls}, 
     fnInv1 = makeRadiusFunction[{f1}, {x, x0, xm}];
     fnInv2 = makeRadiusFunction[{f2}, {x, xm, x1}];
     Which[fnInv1 === $Failed, Throw[#] &[{$Failed, 1}], 
      fnInv2 === $Failed, Throw[{$Failed, 2}], 
      True, \[Pi] stp Sum[(-fnInv1[t + stp/2]^2 + 
          fnInv2[t + stp/2]^2), {t, z0, z1 - stp, stp}]]]];
  res]

riemannSum3D[{{f1__}, {f2__}}, {x_Symbol, x0_, xm1_, xm2_, x1_}, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, "Washer"] := 
 Block[{res}, 
  res = Catch[
    Block[{stp = (z1 - z0)/n}, 
     fnInv1 = makeRadiusFunction[{f1}, {x, x0, xm1}];
     fnInv2 = makeRadiusFunction[{f2}, {x, xm2, x1}];
     Which[fnInv1 === $Failed, Throw[#] &[{$Failed, 1}], 
      fnInv2 === $Failed, Throw[{$Failed, 2}], 
      True, \[Pi] stp Sum[(-fnInv1[t + stp/2]^2 + 
          fnInv2[t + stp/2]^2), {t, z0, z1 - stp, stp}]]]];
  res]  
  
  RiemannSum3D[fnLis_List, varlis_List, 
  rnglis : {z_Symbol, z0_, z1_}, n_Integer?Positive, 
    "Washer"] :=
  Block[{res22 = 
    riemannSum3D[fnLis, varlis, rnglis, n, "Washer"]},
  If[MatchQ[res22, {$Failed, 1 | 2}], 
   Message[MakeRadiusFunction::ovrlp, {"inner radius", 
      "outer radius"}[[res22[[2]]]]]];
  res22 /; FreeQ[res22, $Failed]] 
  
  (*========================   For Shells  =======================================*)
  
RiemannSum3D[f : Except[_List], 
   varlis : {var_Symbol, varmin_, varmax_}, n_Integer, "Shell"] /; 
  finitTst[{f}, varlis] := Block[{stp = (varmax - varmin)/n},
  stp \[Pi] Sum[(2 r + stp) f /. var :> (r + stp/2), {r, varmin, 
     varmax - stp, stp}]]  

RiemannSum3D[{f1_, f2_}, {var_Symbol, varmin_?NonNegative, 
   varmax_?Positive}, n_Integer, "Shell"] := 
 Block[{stp = (varmax - varmin)/n}, 
  stp \[Pi] Sum[((2 r + stp) (f1 - f2) /. var :> (r + stp/2)), {r, 
     varmin, varmax - stp, stp}]]

End[ ]  (*End private context*)